package org.processmining.plugins.workloadperiod.ui;

import info.clearthought.layout.TableLayout;

import java.awt.Dimension;

import javax.swing.ButtonGroup;
import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.plugins.workloadperiod.input.WLAnalysis;
import org.processmining.plugins.workloadperiod.main.WLColorTypes;
import org.processmining.plugins.workloadperiod.main.WLSettings;
import org.processmining.plugins.workloadperiod.main.WLShiftTypes;
import org.processmining.plugins.workloadperiod.main.WLSimpleOutput;
import org.processmining.plugins.workloadperiod.main.WLSimpleTypes;

import com.fluxicon.slickerbox.colors.SlickerColors;
import com.fluxicon.slickerbox.components.NiceSlider;
import com.fluxicon.slickerbox.components.NiceSlider.Orientation;
import com.fluxicon.slickerbox.factory.SlickerDecorator;
import com.fluxicon.slickerbox.factory.SlickerFactory;

/**
 * Workload Analysis UserInterface
 * 
 * @author jnakatumba
 * 
 */

public class WLMinerUI {

	private static final long serialVersionUID = 5023180635431663270L;

	/**
	 * The log summary and the current settings. Which steps are shown by the
	 * wizard depends on the current settings. Values shown in some steps depend
	 * on the summary.
	 */
	private WLSettings settings;
	/**
	 * Enumerating all steps.
	 */
	private int introductionStep;
	private int selectionStep;
	private int overviewStep;
	private int qRegressionStep;
	private int bReferenceStep;
	private int simpleRegStep;
	private int nofSteps;

	/**
	 * All steps, using the indices as mentioned above.
	 */
	private myStep[] mySteps;

	private String workLdOption;
	private String processingSpeed;
	private UIPluginContext context;

	private int currentStep;

	public WLMinerUI(final UIPluginContext context) {
		this.context = context;
	}

	public WLSimpleOutput mine(XLog log, WLSettings settings) {

		/**
		 * Remember the settings.
		 */
		this.settings = settings;

		InteractionResult result = InteractionResult.NEXT;
		/**
		 * Number all steps.
		 */
		nofSteps = 0;
		introductionStep = nofSteps++;
		selectionStep = nofSteps++;
		qRegressionStep = nofSteps++;
		bReferenceStep = nofSteps++;
		simpleRegStep = nofSteps++;
		overviewStep = nofSteps++;

		/**
		 * Create all steps.
		 */
		mySteps = new myStep[nofSteps];
		mySteps[introductionStep] = new IntroductionStep();
		mySteps[selectionStep] = new SelectionStep();
		mySteps[qRegressionStep] = new QRegressionStep();
		mySteps[bReferenceStep] = new BReferenceStep();
		mySteps[simpleRegStep] = new SimpleRegStep();
		mySteps[overviewStep] = new OverviewStep();

		currentStep = introductionStep;

		while (true) {
			if (currentStep < 0) {
				currentStep = 0;
			}
			if (currentStep >= nofSteps) {
				currentStep = nofSteps - 1;
			}
			context.log("Current step: " + currentStep);
			result = context.showWizard("Workload Analysis Plugin", currentStep == 0, currentStep == nofSteps - 1,
					mySteps[currentStep]);
			switch (result) {
				case NEXT :
					mySteps[currentStep].readSettings();
					go(1);
					break;
				case PREV :
					go(-1);
					break;
				case FINISHED :
					WLAnalysis miner = new WLAnalysis(context, log);
					WLSimpleOutput output = miner.mine(settings);
					return output;

				default :
					context.getFutureResult(0).cancel(true);

					return null;
			}
		}
	}

	/**
	 * Returns the current settings for the transition system miner.
	 * 
	 * @return The current settings.
	 */
	public WLSettings getSettings() {
		return settings;
	}

	private int go(int direction) {
		currentStep += direction;
		if (currentStep >= 0 && currentStep < nofSteps) {
			if (mySteps[currentStep].precondition()) {
				return currentStep;
			} else {
				return go(direction);
			}
		}
		return currentStep;
	}

	private abstract class myStep extends JPanel {

		private static final long serialVersionUID = 6892655601953727616L;

		public abstract boolean precondition();

		public abstract void readSettings();
	}

	private class IntroductionStep extends myStep {

		private static final long serialVersionUID = -1056226490972073127L;

		public IntroductionStep() {
			initComponents();
		}

		public boolean precondition() {
			return true;
		}

		private void initComponents() {
			double size[][] = { { TableLayout.FILL }, { TableLayout.FILL } };
			setLayout(new TableLayout(size));
			String body = "<p>This wizard will guide you through the process of configuring this analysis plugin.</p>";
			body += "<p>The configuration options for this miner are divided into two categories:<ol>";
			body += "<li>options for calculating the workload based on the how busy perspective, and </li>";
			body += "<li>options for calculating the workload based on the queue length perspective.</li></ol>";
			body += "The wizard will allow you to choose the inputs based on these two perspectives.</p>";
			add(SlickerFactory.instance().createLabel("<html><h1>Introduction</h1>" + body), "0, 0, l, t");
		}

		public void readSettings() {
		}
	}

	private class SelectionStep extends myStep {

		private static final long serialVersionUID = 6933657976761947420L;

		public SelectionStep() {
			initComponents();
		}

		public boolean precondition() {
			return true;
		}

		private void initComponents() {
			double size[][] = { { 100, TableLayout.FILL }, { 50, 30, 20, 20, 20, 30, 20, 20, TableLayout.FILL } };
			setLayout(new TableLayout(size));
			add(SlickerFactory.instance().createLabel("<html><h2>Inputs for the workload calculation plug-in</h2>"), "0, 0, 1, 0");

			howBusy = SlickerFactory.instance().createRadioButton("Workload Based on How Busy");
			queueLength = SlickerFactory.instance().createRadioButton("Workload Based on Queue Length");
			ButtonGroup typeGroup = new ButtonGroup();
			typeGroup.add(howBusy);
			typeGroup.add(queueLength);

			add(SlickerFactory.instance().createLabel("<html><h3>Please select the desired Workload Perspective</h3>"),
					"0, 1, 1, 1");
			add(howBusy, "0, 2, 1, 2");
			add(queueLength, "0, 3, 1, 3");
			howBusy.setSelected(true);

			JScrollPane jScrollPane2 = new javax.swing.JScrollPane();
			SlickerDecorator.instance().decorate(jScrollPane2, SlickerColors.COLOR_BG_3, SlickerColors.COLOR_FG,
					SlickerColors.COLOR_BG_1);

			add(SlickerFactory.instance().createLabel(
					"<html><h3>Please select the desired processing speeds Perspective</h3>"), "0, 5, 1, 5");
			waitingTimeButton = SlickerFactory.instance().createRadioButton(
					"Processing Speeds denoted by waiting times");
			serviceTimeButton = SlickerFactory.instance().createRadioButton(
					"Processing Speeds denoted by service times");

			ButtonGroup limitGroup = new ButtonGroup();
			limitGroup.add(serviceTimeButton);
			limitGroup.add(waitingTimeButton);
			add(serviceTimeButton, "0, 6, 1, 6");
			add(waitingTimeButton, "0, 7, 1, 7");
			serviceTimeButton.setSelected(true);

		}

		private javax.swing.JRadioButton howBusy;
		private javax.swing.JRadioButton queueLength;
		private javax.swing.JRadioButton waitingTimeButton;
		private javax.swing.JRadioButton serviceTimeButton;

		// End of variables declaration

		public void readSettings() {
			if (queueLength.isSelected()) {
				workLdOption = "Queue Length";
				settings.setWorkLoadType(workLdOption);

			} else if (howBusy.isSelected()) {
				workLdOption = "How Busy";
				settings.setWorkLoadType(workLdOption);
			}
			if (serviceTimeButton.isSelected()) {
				processingSpeed = "Service Times";
				settings.setSpeedType(processingSpeed);

			} else if (waitingTimeButton.isSelected()) {
				processingSpeed = "Waiting Times";
				settings.setSpeedType(processingSpeed);

			}

		}
	}

	private class BReferenceStep extends myStep {
		private static final long serialVersionUID = 6933657976761947420L;

		public BReferenceStep() {
			initComponents();
		}

		public boolean precondition() {
			if (settings.getWorkLoadType().equals("How Busy") == true) {
				return true;
			}
			return false;

		}

		private void initComponents() {
			
			double size[][] = { { 100, TableLayout.FILL }, { 50, 30, 20, 20, 20, 30, 20, 20, TableLayout.FILL } };
			setLayout(new TableLayout(size));
			
			jSlider = SlickerFactory.instance().createNiceIntegerSlider("<html><h3>Select the hours:</h3>", 0, 24, 1, Orientation.HORIZONTAL);
			jSlider.setPreferredSize(new Dimension(400, 15));

			add(jSlider, "0, 8, 1 ,8");
			
			Object[] eventTypes = { WLShiftTypes.START.getLabel(), WLShiftTypes.COMPLETE.getLabel(),
					WLShiftTypes.STARTCOMPLETE.getLabel() };

			refComboBox = SlickerFactory.instance().createComboBox(eventTypes);
			loadComboBox = SlickerFactory.instance().createComboBox(eventTypes);
			loadComboBox.setSize(30, 30);
			refComboBox.setSize(30,30);
			
			add(SlickerFactory.instance().createLabel("<html><h3>Select the load event type:</h3>"), "0, 3, 1, 3");
			add(loadComboBox, "0, 4, 1, 4");
			
			
			add(SlickerFactory.instance().createLabel("<html><h3>Select the reference event type:</h3>"), "0, 5, 1, 5");
			add(refComboBox, "0, 6, 1, 6");
			
			add(SlickerFactory.instance().createLabel("<html><h3>Select the regression type:</h2>"), "0, 0, 1, 0");
			
			simpleReg = SlickerFactory.instance().createRadioButton("Simple Regression");
			
			ButtonGroup typeGroup = new ButtonGroup();
			typeGroup.add(simpleReg);
			add(simpleReg, "0, 1, 1, 1");
			simpleReg.setSelected(true);
	
		}

		private javax.swing.JComboBox refComboBox;
		private javax.swing.JComboBox loadComboBox;
		private javax.swing.JRadioButton simpleReg;
		private Integer shiftPeriod = 0;
		private String displayChoice;
		private NiceSlider jSlider;

		public void readSettings() {
			int indexR = refComboBox.getSelectedIndex();
			switch (indexR) {
				case 0 :
					settings.setRefEventType(WLShiftTypes.START);
					break;
				case 1 :
					settings.setRefEventType(WLShiftTypes.COMPLETE);
					break;
				case 2 :
					settings.setRefEventType(WLShiftTypes.STARTCOMPLETE);
					break;
				default :
					break;
			}

			int indexL = loadComboBox.getSelectedIndex();
			switch (indexL) {
				case 0 :
					settings.setLoadEventType(WLShiftTypes.START);
					break;
				case 1 :
					settings.setLoadEventType(WLShiftTypes.COMPLETE);
					break;
				case 2 :
					settings.setLoadEventType(WLShiftTypes.STARTCOMPLETE);
					break;
				default :
					break;
			}
			shiftPeriod = jSlider.getSlider().getValue();
			settings.setShiftHours(shiftPeriod);

			if (simpleReg.isSelected()) {
				displayChoice = "Simple Regression";
				settings.setRegressionType(displayChoice);

			}
		}
	}

	private class QRegressionStep extends myStep {
		private static final long serialVersionUID = 6933657976761947420L;

		public QRegressionStep() {
			initComponents();
		}

		public boolean precondition() {
			if (settings.getWorkLoadType().equals("Queue Length") == true) {
				return true;
			}
			return false;

		}

		private void initComponents() {

			double size[][] = { { TableLayout.FILL, 20, TableLayout.FILL }, { 50, 30, TableLayout.FILL, 30 } };
			setLayout(new TableLayout(size));

			add(SlickerFactory.instance().createLabel("<html><h2>Select the Reference Event Types</h2>"), "0, 0, 1, 0");

			DefaultListModel listModel = new DefaultListModel();
			listModel.addElement(WLSimpleTypes.ALLRESOURCESONETASK.getLabel());
			listModel.addElement(WLSimpleTypes.ALLRESOURCESTASKS.getLabel());
			listModel.addElement(WLSimpleTypes.ALLTASKSONERESOURCE.getLabel());
			listModel.addElement(WLSimpleTypes.ONERESOURCEONETASK.getLabel());

			simpleDisplayList = new javax.swing.JList(listModel);
			simpleDisplayList.setFixedCellHeight(40);
			simpleDisplayList.setFixedCellWidth(300);

			simpleDisplayList.setLayoutOrientation(JList.VERTICAL_WRAP);
			simpleDisplayList.setVisibleRowCount(-1);
			simpleDisplayList.setForeground(WLColorTypes.BACKGROUNDCOLOR.getColor());
			simpleDisplayList.setBackground(WLColorTypes.FIELDBACKGROUNDCOLOR.getColor());

			JScrollPane jScrollPane = new javax.swing.JScrollPane();
			SlickerDecorator.instance().decorate(jScrollPane, SlickerColors.COLOR_BG_3, SlickerColors.COLOR_FG,
					SlickerColors.COLOR_BG_1);
			jScrollPane.setPreferredSize(new Dimension(250, 300));
			jScrollPane.setViewportView(simpleDisplayList);
			add(jScrollPane, "0, 2");

			int j = 0;
			int[] indices = new int[WLSimpleTypes.values().length];

			for (; j < WLSimpleTypes.values().length; j++) {
				indices[j] = WLSimpleTypes.values().length + 1; // Dummy value, to be ignored.
			}

			simpleDisplayList.setSelectedIndices(indices);

		}

		private javax.swing.JList simpleDisplayList;

		public void readSettings() {
			settings.getSimpleDisplay().clear();
			Object[] selectedValues = simpleDisplayList.getSelectedValues();
			for (Object object : selectedValues) {
				if (WLSimpleTypes.ONERESOURCEONETASK.getLabel().equals(object)) {
					settings.getSimpleDisplay().add(WLSimpleTypes.ONERESOURCEONETASK);
				}
				if (WLSimpleTypes.ALLRESOURCESONETASK.getLabel().equals(object)) {
					settings.getSimpleDisplay().add(WLSimpleTypes.ALLRESOURCESONETASK);
				}
				if (WLSimpleTypes.ALLRESOURCESTASKS.getLabel().equals(object)) {
					settings.getSimpleDisplay().add(WLSimpleTypes.ALLRESOURCESTASKS);
				}
				if (WLSimpleTypes.ALLTASKSONERESOURCE.getLabel().equals(object)) {
					settings.getSimpleDisplay().add(WLSimpleTypes.ALLTASKSONERESOURCE);
				}

			}

		}
	}

	private class SimpleRegStep extends myStep {
		private static final long serialVersionUID = 6933657976761947420L;

		public SimpleRegStep() {
			initComponents();
		}

		public boolean precondition() {
			if (settings.getWorkLoadType().equals("How Busy") == true) {
				if (settings.getDisplayChoice().equals("Simple Regression") == true) {
					return true;
				}
			}

			return false;

		}

		private void initComponents() {
			double size[][] = { { TableLayout.FILL, 20, TableLayout.FILL }, { 50, 30, TableLayout.FILL, 30 } };
			setLayout(new TableLayout(size));

			add(SlickerFactory.instance().createLabel("<html><h2>Select the Reference Event Types</h2>"), "0, 0, 1, 0");

			DefaultListModel listModel = new DefaultListModel();
			listModel.addElement(WLSimpleTypes.ALLRESOURCESONETASK.getLabel());
			listModel.addElement(WLSimpleTypes.ALLRESOURCESTASKS.getLabel());
			listModel.addElement(WLSimpleTypes.ALLTASKSONERESOURCE.getLabel());
			listModel.addElement(WLSimpleTypes.ONERESOURCEONETASK.getLabel());

			simpleDisplayList = new JList(listModel);
			simpleDisplayList.setFixedCellHeight(40);
			simpleDisplayList.setFixedCellWidth(300);

			simpleDisplayList.setLayoutOrientation(JList.VERTICAL_WRAP);
			simpleDisplayList.setVisibleRowCount(-1);
			simpleDisplayList.setForeground(WLColorTypes.BACKGROUNDCOLOR.getColor());
			simpleDisplayList.setBackground(WLColorTypes.FIELDBACKGROUNDCOLOR.getColor());

			JScrollPane jScrollPane = new javax.swing.JScrollPane();
			SlickerDecorator.instance().decorate(jScrollPane, SlickerColors.COLOR_BG_3, SlickerColors.COLOR_FG,
					SlickerColors.COLOR_BG_1);
			jScrollPane.setPreferredSize(new Dimension(250, 300));
			jScrollPane.setViewportView(simpleDisplayList);
			add(jScrollPane, "0, 2");

			int j = 0;
			int[] indices = new int[WLSimpleTypes.values().length];
			/**
			 * for (WLSimpleTypes simpleDisplays : WLSimpleTypes.values()) { if
			 * (settings.getSimpleDisplay().contains(simpleDisplays)) {
			 * indices[j++] = index; } index++; }
			 */
			for (; j < WLSimpleTypes.values().length; j++) {
				indices[j] = WLSimpleTypes.values().length + 1; // Dummy value, to be ignored.
			}
			simpleDisplayList.setSelectedIndices(indices);

		}

		private javax.swing.JList simpleDisplayList;

		public void readSettings() {
			Object[] selectedValues = simpleDisplayList.getSelectedValues();

			for (Object object : selectedValues) {
				if (WLSimpleTypes.ALLRESOURCESONETASK.getLabel().equals(object)) {
					settings.getSimpleDisplay().add(WLSimpleTypes.ALLRESOURCESONETASK);
				}
				if (WLSimpleTypes.ALLRESOURCESTASKS.getLabel().equals(object)) {
					settings.getSimpleDisplay().add(WLSimpleTypes.ALLRESOURCESTASKS);
				}
				if (WLSimpleTypes.ALLTASKSONERESOURCE.getLabel().equals(object)) {
					settings.getSimpleDisplay().add(WLSimpleTypes.ALLTASKSONERESOURCE);
				}
				if (WLSimpleTypes.ONERESOURCEONETASK.getLabel().equals(object)) {
					settings.getSimpleDisplay().add(WLSimpleTypes.ONERESOURCEONETASK);
				}

			}

		}

	}

	

	private class OverviewStep extends myStep {
		private static final long serialVersionUID = -2853400854055327898L;

		public OverviewStep() {
			initComponents();
		}

		public boolean precondition() {
			makingVisible();
			return true;
		}

		private void initComponents() {
			double size[][] = { { TableLayout.FILL }, { 50, TableLayout.FILL } };
			setLayout(new TableLayout(size));
			add(SlickerFactory.instance().createLabel("<html><h2>Inputs for the Workload plugin</h2>"), "0, 0");

			jTextPane = new JTextPane();
			jTextPane.setContentType("text/html");
			jTextPane.setText("<html><body bgcolor=\"BBBBAA\" text=\"555544\">"
					+ "<br><br><br><br><br><center><font face=\"helvetica,arial,sans-serif\" size=\"+1\">"
					+ "These are the inputs selected for the plug-in...</font></center></body></html>");
			jTextPane.setEditable(false);
			jTextPane.setCaretPosition(0);

			add(jTextPane, "0, 1");

		}

		private String tag(String s, String tag) {
			return "<" + tag + ">" + s + "</" + tag + ">";
		}

		private javax.swing.JTextPane jTextPane;

		private static final String BOLD = "b";
		private static final String PAR = "p";
		private static final String ITALIC = "i";

		public void makingVisible() {
			if ((settings.getWorkLoadType().equals("How Busy") == true)) {
				StringBuffer sb = new StringBuffer("<html><body>");
				sb.append(tag(tag("No of Hours = ", BOLD) + settings.getHours(), PAR));
				sb.append(tag(tag("Reference Event =", BOLD) + settings.getRefShiftTypes(), PAR));
				sb.append(tag(tag("Load Event = ", BOLD) + settings.getLoadShiftTypes(), PAR));
				sb.append(tag(tag("Regression Type = ", BOLD) + settings.getDisplayChoice(), PAR));
				sb.append(tag(tag("Regression Display Options = ", BOLD), PAR));
				if (settings.getDisplayChoice().equals("Simple Regression") == true) {
					if (settings.getSimpleDisplay().contains(WLSimpleTypes.ONERESOURCEONETASK)) {
						sb.append(tag(tag("ONERESOURCEONETASK", ITALIC), PAR));
						sb.append("\n");
					}
					if (settings.getSimpleDisplay().contains(WLSimpleTypes.ALLTASKSONERESOURCE)) {
						sb.append(tag(tag("ALLTASKSONERESOURCE", ITALIC), PAR));
						sb.append("\n");
					}
					if (settings.getSimpleDisplay().contains(WLSimpleTypes.ALLRESOURCESONETASK)) {
						sb.append(tag(tag("ALLRESOURCESONETASK", ITALIC), PAR));
						sb.append("\n");
					}
					if (settings.getSimpleDisplay().contains(WLSimpleTypes.ALLRESOURCESTASKS)) {
						sb.append(tag(tag("ALLRESOURCESTASKS", ITALIC), PAR));
						sb.append("\n");
					}
				} 
				sb.append("\n");
				sb.append("\n");
				sb.append("\n");

				sb.append("</body></html>");
				jTextPane.setText(sb.toString());

			} else if ((settings.getWorkLoadType().equals("Queue Length") == true)) {

				StringBuffer sb = new StringBuffer("<html><body>");

				sb.append(tag(tag("Regression Display Options = ", BOLD), PAR));
				if (settings.getSimpleDisplay().contains(WLSimpleTypes.ONERESOURCEONETASK)) {
					sb.append(tag(tag("ONERESOURCEONETASK", ITALIC), PAR));
					sb.append("\n");
				}
				if (settings.getSimpleDisplay().contains(WLSimpleTypes.ALLTASKSONERESOURCE)) {
					sb.append(tag(tag("ALLTASKSONERESOURCE", ITALIC), PAR));
					sb.append("\n");
				}
				if (settings.getSimpleDisplay().contains(WLSimpleTypes.ALLRESOURCESONETASK)) {
					sb.append(tag(tag("ALLRESOURCESONETASK", ITALIC), PAR));
					sb.append("\n");
				}
				if (settings.getSimpleDisplay().contains(WLSimpleTypes.ALLRESOURCESTASKS)) {
					sb.append(tag(tag("ALLRESOURCESTASKS", ITALIC), PAR));
					sb.append("\n");
				}

				sb.append("\n");
				sb.append("\n");
				sb.append("\n");

				sb.append("</body></html>");
				jTextPane.setText(sb.toString());

			}

		}

		public void readSettings() {

		}
	}

}
